*  Read PIA 1.01
*  -------------
*  By Adam Trionfo
*  August 5, 2010
*  
*  This program display the contents of the APF MP-1000's
*  PIA Register A, which is memory location $2000.  This program
*  was written to help understand how the Joystick Routine in the
*  APF's System ROM works.
*  
*  This program reads the Right and Left Joystick (and all buttons)
*  for each row and column.  The output of this program is displayed in 
*  the upper-left corner of the screen.  The results of memory
*  location can be:  
*  
*               Right Joystick
*         
*         +-----+-----+-----+-----+    Key   
*  Row 0  |  1  |  0  |  4  |  7  |    ---
*         | $FE | $FD | $FB | $F7 |    S = South, or "Down"
*         +-----+-----+-----+-----+    E = East, or "Right"                  
*  Row 1  |  S  |  E  |  N  |  W  |    N = North, or "Up"    
*         | $FE | $FD | $FB | $F7 |    W = West, or "Left"
*         +-----+-----+-----+-----+    F/E = Fire Button or En         
*  Row 2  |  3  |  Cl |  6  |  9  |
*         | $FE | $FD | $FB | $F7 |
*         +-----+-----+-----+-----+         
*  Row 3  |  2  | F/E |  5  |  8  |
*         | $FE | $FD | $FB | $F7 |
*         +-----+-----+-----+-----+         
*
*               Left Joystick
*         
*         +-----+-----+-----+-----+   
*  Row 0  |  1  |  0  |  4  |  7  |
*         | $EF | $DF | $BF | $7F |
*         +-----+-----+-----+-----+                  
*  Row 1  |  S  |  E  |  N  |  W  |    
*         | $EF | $DF | $BF | $7F |
*         +-----+-----+-----+-----+         
*  Row 2  |  3  |  Cl |  6  |  9  |
*         | $EF | $DF | $BF | $7F |
*         +-----+-----+-----+-----+         
*  Row 3  |  2  | F/E |  5  |  8  |
*         | $EF | $DF | $BF | $7F |
*         +-----+-----+-----+-----+         
*
*  Here is an example of how it works.  If row 2 is being read, and
*  the output of $2000 is $BF, then button 6 on the left joystick is
*  being pressed.   
*
*  Release History
*  ---------------
*  
*  Version 1.01 (August 22, 2010)
*  
*     - The only change was that a version number was added
*       to the source code.  The binary is still identical
*       to version 1.0
*
*  Version 1.0 (August 5, 2010)
*  
*     - First Public Release


*  Equates (On-Board Routines)

HEXBYTE   EQU  $0180        ; Full Byte of Hex Code
HEX_LSB   EQU  $0181        ; Hex Four Least Significant Bits
HEX_MSB   EQU  $0182        ; Hex Four Most Significant Bits
ASCII_LSB EQU  $0183        ; Four Least Significant Bits as APF ASCII
ASCII_MSB EQU  $0184        ; Four Most Significant Bits as APF ASCII
STROBENUM EQU  $0185        ; Times to Strobe Zero Through $2002
HOLDMSB   EQU  $0186        ; Storage for temp hold of $2002 4 MSbits
FILLSCRN  EQU  $4296        ; Fill Screen Routine
ONECODE   EQU  $4298        ; Fill Screen with One APF ASCII Code
          
          ORG   $8000       ; Start of APF Cartridge Area
          
*  The BIOS uses the first five bytes of the cartridge  

          FCB   $BB         ; Tell BIOS a cart is present
          FDB   MENUSTR     ; Points to Menu string on cartridge 
          FCB   $31         ; LSB 4 bits is # of Menu Choices
*                             Menu has one choice          
          FCB   $00         ; Number of Players, ALWAYS use ZERO!

*  After a menu choice is made from the APF Menu via the
*  BIOS, then control passes to the program here ($8005).  

*  Clear the screen with "Light Green" 
          LDAA  #$60        ; APF ASCII for "Light Green" 
          JSR   ONECODE     ; Fill Screen with "Light Green"

**********************          
*                    *
*  Main Program Loop *
*                    *
**********************

          JSR   INITPROG    ; Initialize Program
READPIA   JSR   STROBEROW   ; Setup which Button Row to Read
          JSR   GETPIA      ; Read PIA $2000
          JSR   STOREASC    ; Store the ASCII Data on Screen
          JMP   READPIA     ; Continue Running Program Forever 

*****************          
*               *
*  Subroutines  *
*               *
*****************

*  Initialize Program
*  ------------------
*  1) Set default row to read to be last row (2, Fire/En, 5 and 8)
*  2) Place the dollar-sign on screen to represent a hex number
    
INITPROG  STX   STROBE      ; Set Strobe Mask to Row 0
          LDAA  #$03        ; Number of Rows to Loop Through
          STAA  STROBENUM   ; Store Number of Row   
          LDAA  #$64        ; Place an APF ASCII $ in Accum A
          STAA  $0200       ; Place $ in upper-left corner      
          RTS

*  Strobe Joystick Rows 0-3
*  ------------------------
*  1) Get the Joystick Row to Read (Bits 0-3 of $2002)
*  2) Check which row is currently read, if it's 0, then
*     reset it to last row (three)

STROBEROW LDAA  $2002       ; Load Current Button Row to Read (4 LSbits)
          ANDA  #%11110000  ; Keep 4 MSbits
          STAA  HOLDMSB     ; Storage for temp hold of $2002 4 MSbits 
          LDAA  STROBENUM   ; Load the Row to Read    
          CMPA  #$00        ; Start over at Row 3 yet?
          BNE   JOYROW      ; If Zero, then Four Rows Read
          STX   STROBE      ; Reset Strobe Mask to Last Row
          LDAA  #$04        ; Number of Rows to Loop Through
          STAA  STROBENUM   ; Store Number of Row to Read
*                           ; Note: STROBENUM will be 3 before routine ends          
JOYROW    LDX   STROBE      ; Four Mask addresses to Strobe Rows with 
          LDAA  $00,X       ; Get new byte to OR with $2002 
          ORAA  HOLDMSB     ; Update $2002 with new row to read
          STAA  $2002       ; Place Current Row to Read into $2002
          LDAA  STROBENUM   ; Get Number of Rows Read
          DECA              ; Read to read next row down
          STAA  STROBENUM   ; Store Number row number to read (0-3)
          INX               ; Update to next byte to OR with $2002
          RTS

*  Get PIA - Read Column of Joystick
*  --------------------------------- 
*  1) Read the memory location $2000 (this hold joystick column)
*  2) Store Joystick column info in byte format that has Most
*     Significant as zero for preperation for later ASCII
*     conversion.
     
GETPIA    LDAA  $2000       ; Get data from PIA
          STAA  HEXBYTE     ; Store the whole hex Byte
          ANDA  #%00001111  ; MASK.  Keep 4 LSb
          STAA  HEX_LSB     ; Store Four Least Significant Bits
          LDAA  HEXBYTE     ; Get complete Hex Byte again
          LSRA              ; Right Shift Four Times to Keep 
          LSRA              ; Four Most Significant Bits
          LSRA
          LSRA
          STAA  HEX_MSB     ; Keep 4 MSbs of Hex Byte
          BSR   HEXTOASC    ; Convert 4 MSbits of Hex Byte to APF ASCII
          STAA  ASCII_MSB   ; Store 4 MSbits as APF ASCII 
          LDAA  HEX_LSB     ; Get the 4 LSbits  
          BSR   HEXTOASC    ; Convert 4 MSbits of Hex Byte to APF ASCII
          STAA  ASCII_LSB   ; Store 4 MSbits as APF ASCII
          RTS
          
*  Hex to APF ASCII Conversion
*  ---------------------------
*  1) Convert a Hex digit $00-0F already in Accum A to APF 
*     ASCII.  The Hex digit's four Most Significant Bits must
*     already be zero.
           
HEXTOASC  CMPA  #$09
          BHI   NOT_0TO9    ; Branch if $0A-$0F
          ADDA  #$39        ; No, it's $0A-0F, Place offset for numbers
NOT_0TO9  ADDA  #$37        ; Convert from Hex to APF ASCII
          RTS


*  Store APF ASCII data to Screen
*  ------------------------------
*  1) Store APF ASCII Data to Upper-Left part of Screen
  
STOREASC  LDAA  ASCII_MSB   ; 
          STAA  $0201       ; Store after $
          LDAA  ASCII_LSB   ; 
          STAA  $0202       ; Store after MSB
          RTS

 
*  Cartridge Menu String
*  ---------------------
*  
*  The data required by the BIOS for the Main Menu is here.

MENUSTR   FCB   $EA         ; 10 spaces before Program Name string
          FCC   "READ PIA  1.0"  ; Program Name String          
          FCB   $E9         ; 9 spaces after text (EOL)
          FCB   $E9         ; 9 spaces (Before Author)
          FCC   "BY ADAM TRIONFO"  ; Author
          FCB   $E8         ; 8 spaces (After Author)
          FCB   $EA         ; 10 spaces (Before Date)          
          FCC   "AUG  5, 2010" ; Date
          FCB   $EA         ; 10 spaces (After Date)
          FCC   "1. RUN PROGRAM"
          FCB   $CF         ; Control Byte - 15 fill-bytes
          FCB   $60         ; Fill-Byte is Light Green 
          FCB   $C3         ; Control Byte - 3 fill-bytes
          FCB   $60         ; Fill-Byte is Light Green
*  One Full Black Line of 32 Characters
          FCB   $DF         ; Control Byte - 15 fill-bytes
          FCB   $80         ; Fill-Byte is Black          
          FCB   $DF         ; Control Byte - 15 fill-bytes
          FCB   $80         ; Fill-Byte is Black
          FCB   $C2         ; Control Byte - 2 fill-bytes
          FCB   $80         ; Fill-Byte is Black
          FCB   $FF         ; Control Byte - End of String

*  Mask for Moving Through $2002
STROBE    FCB   %00001110   ; Row 0 (Bit 0 is Low)   
          FCB   %00001101   ; Row 1 (Bit 1 is Low)
          FCB   %00001011   ; Row 2 (Bit 2 is Low)
          FCB   %00000111   ; Row 3 (Bit 3 is Low)

CARTEND   FILL  $FF,$9000-CARTEND ; Pad with $FF for a 4K Cart

* End of Program